.include "tn13def.inc"

.def	Null			=r2		;Register mit Wert $00
.def	RandomL			=r3
.def	RandomH			=r4


.def	tmp				=r15	;Interrupt-Temp Register

.def	Temp 			=r16  	;Temp Register
.def	TempH			=r17	;TempHigh Register
.def	Temp3			=r18	;Temp3 Register
.def	Prescaler1		=r19
.def	Prescaler2		=r20
.def	PWMValue		=r21
.def	RandomVal		=r22

;StatusRegister:
.equ	TickFlag		=0		;set each timer compare
.equ	TXFlag			=1		;set if TX-routine is running
.equ	TransmitFlag	=2		;set if TX-routine is enabled
.equ 	SetFlag			=3

.equ	ResetFlags		=$60
.equ	Status0			=$61


;**********************************************************************************
.org $00			;(Reset Handler)
 
 rjmp	RESET 		;(Reset Handler)
 rjmp	RESET		;External Interrupt0 Handler
 rjmp	RESET		;Pin change Interrupt Handler
 rjmp	SysTime		;Timer0 Overflow Handler
 rjmp	RESET		;EEPROM Ready Handler
 rjmp	RESET		;Analog Comparator Handler
 rjmp	RESET		;Timer0 Compare A Handler
 rjmp	RESET		;Timer0 Compare B Handler
 rjmp	RESET		;Watchdog Overflow Handler
 rjmp	RESET		;Analog / Digital Converter Handler

RESET:
 cli

;Stackpointer initialisieren
 ldi	Temp, low (RAMEND);
 out	SPL, Temp

;Watchdog initialisieren
 wdr
 in 	Temp, WDTCR
 ori	Temp, (1<<WDCE) | (1<<WDE)
 out	WDTCR, Temp
 ldi	Temp, 7											;Watchdog Time-Out ~ 2 sec.
 out	WDTCR, Temp

 clr	Null

;erase complete SRAM 
 clr	XH
 ldi	XL, $60
 clr	Temp
 ldi	TempH, 64

_CLSR:
 st		X+, Temp
 dec	TempH
 brne	_CLSR


 ldi	Temp, 0b10011010		 
 mov	RandomL, Temp			;load Seed into Random RegL

 in 	Temp, MCUSR
 sts	ResetFlags, Temp
 clr	Temp
 out	MCUSR, Temp

;Ports initialisieren
; PortB0: (I)		ISP MOSI
; PortB1: (O)		ISP MISO / PWM out OCR0B
; PortB2: (I)		
; PortB3: (I)		
; PortB4: (I)		
; PortB5: (I)		RESET

 ldi	Temp,  0b00000010
 ldi 	TempH, 0b00111101
 out	DDRB, Temp
 out 	PortB, TempH										


;8 Bit Timer0 	
 ldi	Temp, 0b00100011		;Fast PWM
 out	TCCR0A, Temp
 ldi	Temp, 0b00000001		;Prescaler 8	
 out	TCCR0B, Temp
 
 ldi	Temp, 5					;TestValue
 out	OCR0B, Temp

 ldi	Temp, 0b00000010		;Timer0 OverFlow Interrupt enable
 out	TIMSK0, Temp
 
 ldi	Prescaler1, 30
 ldi	Prescaler2, 40

 sei

 
;*********** H A U P T P R O G R A M M ***********


BEGIN: 
 wdr
 
 rjmp	BEGIN					;loop to begin


; ******** INTERRUPT SERVICE ROUTINEN *********

;internal timer generation
SysTime:
 in		tmp, SREG
 push	tmp
 push	Temp
 
 dec	Prescaler1
 brne	_ST2
 ldi	Prescaler1, 30				;fading speed
 
 rcall 	Random

 dec	Prescaler2
 brne	_STX
 ldi	Prescaler2, 40				;interval for taking new random values

 mov	RandomVal,  RandomL
 cpi	RandomVal, 10
 brcc	_STX
 ldi	RandomVal, 10				;minimum brightness

_STX:
 cpi	RandomVal, 50				;duty cycle: more light, less darkness (127: 50/50)
 brcc	_ST0

 cp		PWMValue, RandomVal
 breq	_ST1
 brcs	_ST1
 dec	PWMValue
 rjmp	_ST1

_ST0:
 cpi	PWMValue, 255
 breq	_ST1
 inc	PWMValue
 
_ST1: 
 out	OCR0B, PWMValue
 

_ST2:
 pop	Temp
 pop	tmp
 out	SREG, tmp
 reti


;************************
; S U B R O U T I N E S *
;************************


;generates 16 bit random value (16Bit SR with feedback on Bit 0, 2, 11, 15)
RANDOM:
 mov	Temp, RandomL		;exor feedback outputs for SR input
 andi	Temp, 0b00000001
 lsl	Temp
 lsl	Temp
 eor	Temp, RandomL
 andi	Temp, 0b00000100
 swap 	Temp
 lsl	Temp
 mov	TempH, RandomH
 andi	TempH, 0b00001000
 swap	TempH
 eor	TempH, RandomH
 andi	TempH, 0b10000000
 eor	TempH, Temp
 rol	TempH		
 rol	RandomL
 rol	RandomH
 ret





